'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function openJetbrainsIde(directory) {
    _fs2.default.readdir(directory, function (err, items) {
        var projectStat = items.reduce(function (acc, f) {
            return {
                projectFile: acc.projectFile || (isProjectFile(f) ? f : null),
                hasWebStormFiles: acc.hasWebStormFiles || isWebStormFile(f),
                hasPhpStormFiles: acc.hasPhpStormFiles || isPhpStormFile(f)
            };
        }, {
            projectFile: null,
            hasWebStormFiles: false,
            hasPhpStormFiles: false
        });

        var appName = '';
        if (projectStat.hasPhpStormFiles) {
            appName = 'PhpStorm';
        } else if (projectStat.hasWebStormFiles) {
            appName = 'WebStorm';
        } else {
            appName = 'IntelliJ';
        }

        getJetBrainsApp(appName, function (err, application) {
            return openByApplication(err, application, projectStat.projectFile);
        });
    });

    function isProjectFile(file) {
        var projectFilePatterns = [/pom\.xml/, /\.ipr/, /build\.xml/];
        var isProjectFile = false;

        projectFilePatterns.forEach(function (p) {
            return isProjectFile = isProjectFile || p.test(file);
        });

        return isProjectFile;
    }

    function isWebStormFile(file) {
        return ['package.json', 'bower.json', 'gulpfile.json', 'gruntfile.json'].includes(file);
    }

    function isPhpStormFile(file) {
        return ['composer.json'].includes(file);
    }

    function openByApplication(error, application, project) {

        if (error) {
            console.error(_chalk2.default.bold.red(error.message));
            process.exit(1);
        }

        var filename = null;
        if (application === '.idea') {
            filename = directory;
        } else {
            filename = project || directory;
        }

        console.log(_chalk2.default.green('opening with ' + application));
        _shelljs2.default.exec('open -a "' + application + '" "' + filename + '"', function (code, stdout, stderr) {
            if (code !== 0) {
                console.error(_chalk2.default.bold.red('' + stdout));
                console.error(_chalk2.default.bold.red('' + stderr));
                _shelljs2.default.exit(code);
            }
        });
    }

    function getJetBrainsApp(appName, callback) {
        var application = _shelljs2.default.ls('-d', '/Applications/' + appName + '*').pop();

        if (!application) {
            callback(new Error('Unable to find ' + appName + ' application'));
        } else {
            callback(null, application.trim());
        }
    }
}

module.exports = openJetbrainsIde;